@@ -1,6 +1,6 @@
 /*
  *
- * $Id: postgre.c, v 0.9 2001/04/06 23:22:45 kaiser Exp $
+ * $Id: ipchains.c,v 1.4 2003/11/14 19:51:29 friedl Exp $
  *
  * postgresql backend to ipac
  * Copyright (C) 2001 Al Zaharov
@@ -60,22 +60,14 @@ static int flush_acc_chains(void);
 int ipchains_ipac_init(int flag);
 int ipchains_ipac_set(rule_type **firstrule, int first);
 int ipchains_ipac_read(rule_type **firstrule);
-int ipchains_ipac_deny(char *login);
-int ipchains_ipac_accept(char *login);
 int ipchains_ipac_check(void);
-int ipchains_ipac_remove_user(char *login);
-int ipchains_ipac_alarm();
 
 static const acc_agent_t interface_entry = {
 	"ipchains",
 	ipchains_ipac_init,
 	ipchains_ipac_set,
 	ipchains_ipac_read,
-	ipchains_ipac_deny,
-	ipchains_ipac_accept,
 	ipchains_ipac_check,
-	ipchains_ipac_remove_user,
-	ipchains_ipac_alarm,
 };
 
 const acc_agent_t *ipac_ag_interface_ipchains() {
@@ -440,16 +432,6 @@ setup_chains()
 	parse_ports(NULL, fw.ipfw.fw_dpts, fw.ipfw.fw_proto);
 	ipfwc_create_chain(CH_INNAME);
 	ipfwc_create_chain(CH_OUTNAME);
-	if (!is_classic) {
-		ipfwc_create_chain(CH_CTRL_IN);
-		ipfwc_create_chain(CH_CTRL_OUT);
-		strcpy(fw.label, CH_CTRL_IN);
-		ipfwc_delete_entry("input", &fw);
-		ipfwc_append_entry("input", &fw);
-		strcpy(fw.label, CH_CTRL_OUT);
-		ipfwc_delete_entry("output", &fw);
-		ipfwc_append_entry("output", &fw);
-	}
 	strcpy(fw.label, CH_INNAME);
 	ipfwc_delete_entry("input", &fw);
 	ipfwc_append_entry("input", &fw);
@@ -484,11 +466,6 @@ flush_acc_chains(void)
 
 int ipchains_ipac_init(int flag)
 {
-	if (!is_classic) {
-		fprintf(stderr, "Neoclassic mode does not supported with "
-						    "ipchains, aborting\n");
-		exit(1);
-	}
 	return 0;
 };
 
@@ -511,8 +488,8 @@ check_inverse(char *src)
                         	return 0;
 
                         if (memcmp(src, "!", 1) == 0) {
-				fprintf(stderr, "Multiple `!' flags not 
-								    allowed");
+				fprintf(stderr, 
+					"Multiple `!' flags not allowed");
 				exit(1);
 			}
                         return 1;
@@ -663,117 +640,8 @@ int ipchains_ipac_read(rule_type **firstrule)
 	return retval;
 }
 
-int 
-ipchains_ipac_accept(char *login)
-{
-	static struct ip_fwuser fw;
-	raw_rule_type *dat;
-	unsigned int nsaddrs = 0;
-	struct in_addr *saddrs = NULL, *daddrs = NULL;
-	
-	access_agent->get_raw_list("ipchains", login, &dat);
-	while(dat)
-	{
-		parse_hostnetworkmask(dat->snet, 
-				&saddrs, &(fw.ipfw.fw_smsk), &nsaddrs);	
-		parse_hostnetworkmask(dat->dnet, &daddrs,
-				&(fw.ipfw.fw_dmsk), &nsaddrs);
-		if (!dat->protocol[0]=='\0')
-			fw.ipfw.fw_proto = parse_protocol(dat->protocol);
-		if (fw.ipfw.fw_proto != IPPROTO_ICMP)
-		{
-			parse_ports(dat->sport, fw.ipfw.fw_spts, 
-							    fw.ipfw.fw_proto);
-			parse_ports(dat->dport, fw.ipfw.fw_dpts, 
-							    fw.ipfw.fw_proto);
-		}
-		strcpy(fw.label, "");
-		fw.ipfw.fw_src.s_addr = saddrs[0].s_addr;
-		fw.ipfw.fw_dst.s_addr = daddrs[0].s_addr;
-		
-		if (!memcmp(dat->dest, "out", 3))
-		{
-			strcpy(fw.label, "REJECT");
-			ipfwc_delete_entry(CH_CTRL_OUT, &fw);
-			strcpy(fw.label, "");
-			ipfwc_delete_entry(CH_CTRL_OUT, &fw);
-			ipfwc_append_entry(CH_CTRL_OUT, &fw);
-		} else {
-			strcpy(fw.label, "REJECT");
-			ipfwc_delete_entry(CH_CTRL_IN, &fw);
-			strcpy(fw.label, "");
-			ipfwc_delete_entry(CH_CTRL_IN, &fw);
-			ipfwc_append_entry(CH_CTRL_IN, &fw);
-		}
-		dat=dat->next;
-	}
-	return 0;
-}
-
-int 
-ipchains_ipac_deny(char *login)
-{
-	static struct ip_fwuser fw;
-	raw_rule_type *dat;
-	unsigned int nsaddrs = 0;
-	struct in_addr *saddrs = NULL, *daddrs = NULL;
-	
-	access_agent->get_raw_list("ipchains", login, &dat);
-	while(dat)
-	{
-		parse_hostnetworkmask(dat->snet, 
-				&saddrs, &(fw.ipfw.fw_smsk), &nsaddrs);	
-		parse_hostnetworkmask(dat->dnet, &daddrs,
-				&(fw.ipfw.fw_dmsk), &nsaddrs);
-		if (!dat->protocol[0]=='\0')
-			fw.ipfw.fw_proto = parse_protocol(dat->protocol);
-		if (fw.ipfw.fw_proto != IPPROTO_ICMP)
-		{
-			parse_ports(dat->sport, fw.ipfw.fw_spts, 
-							    fw.ipfw.fw_proto);
-			parse_ports(dat->dport, fw.ipfw.fw_dpts, 
-							    fw.ipfw.fw_proto);
-		}
-		strcpy(fw.label, "");
-		fw.ipfw.fw_src.s_addr = saddrs[0].s_addr;
-		fw.ipfw.fw_dst.s_addr = daddrs[0].s_addr;
-		
-		if (!memcmp(dat->dest, "out", 3))
-		{
-			strcpy(fw.label, "");
-			ipfwc_delete_entry(CH_CTRL_OUT, &fw);
-			strcpy(fw.label, "REJECT");
-			ipfwc_delete_entry(CH_CTRL_OUT, &fw);
-			ipfwc_append_entry(CH_CTRL_OUT, &fw);
-		}
-		else
-		{
-			strcpy(fw.label, "");
-			ipfwc_delete_entry(CH_CTRL_IN, &fw);
-			strcpy(fw.label, "REJECT");
-			ipfwc_delete_entry(CH_CTRL_IN, &fw);
-			ipfwc_append_entry(CH_CTRL_IN, &fw);
-		}
-		dat=dat->next;
-	}
-	return 0;
-}
-
 int
 ipchains_ipac_check(void)
 {
 	return 0;
 }
-
-int
-ipchains_ipac_remove_user(char *login)
-{
-	return 0;
-}
-
-/* not implemented */
-int
-ipchains_ipac_alarm(void)
-{
-	return 1;
-}
