@@ -1,9 +1,9 @@
 /*
  *
- * $Id: postgre.c, v 0.9 2001/04/06 23:22:45 kaiser Exp $
+ * $Id: iptables.c,v 1.7 2004/06/27 22:08:54 friedl Exp $
  *
  * postgresql backend to ipac
- * Copyright (C) 2001-2002 Al Zaharov
+ * Copyright (C) 2001-2003 Al Zakharov
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
  *
 */
 
+#include <getopt.h>
 #include <string.h>
 #include <stdio.h>
 #include <dirent.h>
@@ -44,6 +45,7 @@
  * #include <ctype.h> for isalnum() in parse_interface()
  */
 #include <ctype.h>
+#include <linux/netfilter_ipv4/ipt_mark.h>
 #include "ipac.h"
 #include "libiptc.h"
 #include "../../lib/libnet.h"
@@ -78,6 +80,17 @@
 			| FMT_NUMERIC | FMT_NOTABLE)
 #define FMT(tab,notab) ((format) & FMT_NOTABLE ? (notab) : (tab))
 
+#define OPTION_OFFSET 256
+
+static struct option original_opts[] = {
+	{ "match", 1, 0, 'm' },
+	{ 0 }
+};
+
+
+static struct option *opts = original_opts;
+static unsigned int global_option_offset = 0;
+
 extern char *authhost;
 
 /* - T.Mohan 5/7/2001
@@ -210,28 +223,18 @@ struct iptables_target *iptables_targets = NULL;
 extern void register_match(struct iptables_match *me);
 extern void register_target(struct iptables_target *me);
 
-static int clear_runfile(char *serv_name, int greedy);
-
 /* plain file ipac interface entries */
 int iptables_ipac_init(int flag);
 int iptables_ipac_set(rule_type **firstrule, int first);
 int iptables_ipac_read(rule_type **firstrule);
-int iptables_ipac_deny(char *login);
-int iptables_ipac_accept(char *login);
 int iptables_ipac_check(void);
-int iptables_ipac_remove_user(char *login);
-int iptables_ipac_alarm();
 
 static const acc_agent_t interface_entry = {
 	"iptables",
 	iptables_ipac_init,
 	iptables_ipac_set,
 	iptables_ipac_read,
-	iptables_ipac_deny,
-	iptables_ipac_accept,
 	iptables_ipac_check,
-	iptables_ipac_remove_user,
-	iptables_ipac_alarm,
 };
 
 const acc_agent_t *ipac_ag_interface_iptables() {
@@ -256,8 +259,7 @@ exit_error(enum exittype status, char *msg, ...)
 //	if (status == PARAMETER_PROBLEM)
 //		exit_tryhelp(status);
 	if (status == VERSION_PROBLEM)
-		fprintf(stderr,
-	"Perhaps iptables or your "
+		fprintf(stderr,	"Perhaps iptables or your "
 					    "kernel needs to be upgraded.\n");
 	exit(status);
 }
@@ -404,10 +406,8 @@ parse_interface(const char *arg, char *vianame, unsigned char *mask)
 int
 check_inverse_type(char* src)
 {
-	if (src)
- {
-		if (memcmp(src, "!", 1) == 0)
- {
+	if (src) {
+		if (memcmp(src, "!", 1) == 0) {
 			int slen = strlen(src);
 
 			//strip the "!"
@@ -779,13 +779,6 @@ register_match(struct iptables_match *me)
 {
 	struct iptables_match **i;
 
-/*
-	if (strcmp(me->version, program_version) != 0) {
-		fprintf(stderr, "%s: match `%s' v%s (I'm v%s).\n",
-			program_name, me->name, me->version, program_version);
-		exit(1);
-	}
-*/
 	if (find_match(me->name, DONT_LOAD)) {
 		fprintf(stderr, "%s: match `%s' already registered.\n",
 			"fetchipac??", me->name);
@@ -1170,13 +1163,36 @@ read_iptables(struct runfile_line_type *runfile, rule_type **firstrule)
 	return 0;
 }
 
+static struct option *
+merge_options(struct option *oldopts, const struct option *newopts,
+		unsigned int *option_offset)
+{
+	unsigned int num_old, num_new, i;
+	struct option *merge;
+
+	for (num_old = 0; oldopts[num_old].name; num_old++);
+	for (num_new = 0; newopts[num_new].name; num_new++);
+
+	global_option_offset += OPTION_OFFSET;
+	*option_offset = global_option_offset;
+
+	merge = malloc(sizeof(struct option) * (num_new + num_old + 1));
+	memcpy(merge, oldopts, num_old * sizeof(struct option));
+	for (i = 0; i < num_new; i++) {
+		merge[num_old + i] = newopts[i];
+		merge[num_old + i].val += *option_offset;
+	}
+	memset(merge + num_old + num_new, 0, sizeof(struct option));
+
+	return merge;
+}
+
 /*
  * Prepare ipt entry for such a funcs as insert, delete, append, replace rule
  * 
  */
 static int
-prepare_entry (char *chain, char *saddr, char *sport, char *daddr, char *dport,
-		char *proto, char *targ, char *iface, struct ipt_entry **e)
+prepare_entry (raw_rule_type *d, struct ipt_entry **e)
 {
 	struct ipt_entry fw;
 	unsigned int naddrs = 0;
@@ -1186,24 +1202,29 @@ prepare_entry (char *chain, char *saddr, char *sport, char *daddr, char *dport,
 	struct iptables_target *t;
 	size_t size;
 	int inverse;
+	int c,argc;
+	int invert = 0;
 
-	memset(&fw, 0, sizeof(fw));
+	bzero(&fw, sizeof(fw));
 
-	if (!strcmp(proto, "all"))
-		proto[0]='\0';
+	if (verbose>2)
+		printf("preparing entry for '%s' chain\n", d->dest);
 
-	if (iface && strlen(iface)>1) {
-		if ((!strncmp(chain+strlen(chain)-2, "~o", 2)) ||
-				(!strncmp(chain+strlen(chain)-3, "~fi", 3)) ||
-				(!strncmp(chain+strlen(chain)-4, "~c_o", 4)) ||
-			        (!strncmp(chain+strlen(chain)-5, "~c_fi", 5))) {
-			inverse = check_inverse_type(iface);
-			parse_interface(iface, fw.ip.iniface, fw.ip.iniface_mask);
+	if (!strcmp(d->protocol, "all"))
+		d->protocol[0]='\0';
+
+	if (d->iface && strlen(d->iface)>1) {
+		if ((!strncmp(d->dest+strlen(d->dest)-2, "~o", 2)) ||
+				(!strncmp(d->dest+strlen(d->dest)-3, "~fi", 3)) ||
+				(!strncmp(d->dest+strlen(d->dest)-4, "~c_o", 4)) ||
+			        (!strncmp(d->dest+strlen(d->dest)-5, "~c_fi", 5))) {
+			inverse = check_inverse_type(d->iface);
+			parse_interface(d->iface, fw.ip.iniface, fw.ip.iniface_mask);
 	                fw.ip.invflags |= (inverse ? IPT_INV_VIA_IN : 0);
 			fw.nfcache = NFC_IP_IF_IN;
 		} else {
-			inverse = check_inverse_type(iface);
-			parse_interface(iface, fw.ip.outiface, fw.ip.outiface_mask);
+			inverse = check_inverse_type(d->iface);
+			parse_interface(d->iface, fw.ip.outiface, fw.ip.outiface_mask);
 	                fw.ip.invflags |= (inverse ? IPT_INV_VIA_OUT : 0);
 			fw.nfcache = NFC_IP_IF_OUT;
 		}
@@ -1220,26 +1241,26 @@ prepare_entry (char *chain, char *saddr, char *sport, char *daddr, char *dport,
 		t->used = 0;
 	}
 	
-	if (!iptc_is_chain(chain, handle)) {
-		fprintf(stderr, "%s is not a chain\n", chain);
+	if (!iptc_is_chain(d->dest, handle)) {
+		fprintf(stderr, "%s is not a chain\n", d->dest);
 		return (1);
 	}
-	
-	if (strlen(saddr)>2) {
-		if (check_inverse_type(saddr))
+
+	if (strlen(d->snet)>2) {
+		if (check_inverse_type(d->snet))
 			fw.ip.invflags |= IPT_INV_SRCIP;
-		parse_hostnetworkmask(saddr, &addrs, &(fw.ip.smsk), &naddrs);
+		parse_hostnetworkmask(d->snet, &addrs, &(fw.ip.smsk), &naddrs);
 		if (naddrs>1)
 			exit_error(PARAMETER_PROBLEM, 
 				"Incorrect rule: more than 1 source address\n");
 		fw.ip.src.s_addr = addrs[0].s_addr;
 		fw.nfcache |= NFC_IP_SRC;
 	}
-
-	if (strlen(daddr)>2) {
-		if (check_inverse_type(daddr))
+	
+	if (strlen(d->dnet)>2) {
+		if (check_inverse_type(d->dnet))
 			fw.ip.invflags |= IPT_INV_DSTIP;
-		parse_hostnetworkmask(daddr, &addrs, &(fw.ip.dmsk), &naddrs);
+		parse_hostnetworkmask(d->dnet, &addrs, &(fw.ip.dmsk), &naddrs);
 		if (naddrs>1)
 			exit_error(PARAMETER_PROBLEM, "Incorrect rule: more than 1 "
 					    "destination address\n");
@@ -1247,63 +1268,118 @@ prepare_entry (char *chain, char *saddr, char *sport, char *daddr, char *dport,
 		fw.nfcache |= NFC_IP_DST;
 	}
 
-	if ((sport[0]!='\0' || dport[0]!='\0') && proto[0]=='\0')
+	if ((d->sport[0]!='\0' || d->dport[0]!='\0') && d->protocol[0]=='\0')
 		exit_error(PARAMETER_PROBLEM, "Incorrect rule: source or "
-		    "destination port specified while protocol is not\n");
+		    "destination port specified while protocol is not. sport='%s', dport='%s'\n",
+			d->sport, d->dport);
 	
 	/* Loading target /if any/ */
 	target = find_target(IPT_STANDARD_TARGET, LOAD_MUST_SUCCEED);
 	size = sizeof(struct ipt_entry_target) + target->size;
 	target->t = xcalloc(1, size);
 	target->t->u.target_size = size;
-	strcpy(target->t->u.user.name, targ);
+	strcpy(target->t->u.user.name, d->target);
 	target->init(target->t, &fw.nfcache);
 
-	if(check_inverse_type(proto))
+	if(check_inverse_type(d->protocol))
 		fw.ip.invflags |= IPT_INV_PROTO;
 
-	if (proto[0] != '\0') {
-		fw.ip.proto = parse_protocol(proto);
+	if (d->protocol[0] != '\0') {
+		fw.ip.proto = parse_protocol(d->protocol);
 		fw.nfcache |= NFC_IP_PROTO;
 	}
 
-	if (proto[0] != '\0' && proto[0] != 'i') {
-		m = find_proto(proto, LOAD_MUST_SUCCEED, 0);
+	if (d->protocol[0] != '\0' && d->protocol[0] != 'i') {
+		m = find_proto(d->protocol, LOAD_MUST_SUCCEED, 0);
 		size = IPT_ALIGN(sizeof(struct ipt_entry_match)) + m->size;
 		m->m = xcalloc(size, 1);
 		m->m->u.match_size = size;
 		strcpy(m->m->u.user.name, m->name);
 		m->init(m->m, &fw.nfcache);
 
-		if (sport[0]!='\0' || dport[0]!='\0') {
-			if (!strcmp(proto, "tcp")) 
-{
+		if (d->sport[0]!='\0' || d->dport[0]!='\0') {
+			if (!strcmp(d->protocol, "tcp")) {
 				struct ipt_tcp *tcpinfo = (struct ipt_tcp *)m->m->data;
 
-				if (sport[0]!='\0')
+				if (d->sport[0]!='\0')
 					/* - T.Mohan 5/7/2001 */
-					parse_tcp_ports(sport, tcpinfo->spts);
+					parse_tcp_ports(d->sport, tcpinfo->spts);
 
-				if (dport[0]!='\0')
+				if (d->dport[0]!='\0')
 					/* - T.Mohan 5/7/2001 */
-					parse_tcp_ports(dport, tcpinfo->dpts);
+					parse_tcp_ports(d->dport, tcpinfo->dpts);
 			}
 
-			if (!strcmp(proto, "udp")) 
-{
+			if (!strcmp(d->protocol, "udp")) {
 				struct ipt_udp *udpinfo = 
 						(struct ipt_udp *)m->m->data;
-				if (sport[0]!='\0')
+				if (d->sport[0]!='\0')
 					/* - T.Mohan 5/7/2001 */
-					parse_udp_ports(sport, udpinfo->spts);
+					parse_udp_ports(d->sport, udpinfo->spts);
 
-				if (dport[0]!='\0')
+				if (d->dport[0]!='\0')
 					/* - T.Mohan 5/7/2001 */
-					parse_udp_ports(dport, udpinfo->dpts);
+					parse_udp_ports(d->dport, udpinfo->dpts);
 			}
 		}
 	}
 	
+	if (d->extension[2]) {
+		opts = original_opts;
+		optind = 0;
+		global_option_offset = 0;
+
+		d->extension[0] = xstrdup("fetchipac");
+		d->extension[1] = xstrdup("-m");
+	
+		for (argc=0;d->extension[argc];argc++);
+		
+		// parse extension
+		while ((c = getopt_long(argc, d->extension,"-m:", opts, NULL))!= -1) {
+			switch 	(c) {
+				case 'm':
+					m = find_match(optarg, LOAD_MUST_SUCCEED);
+					size = IPT_ALIGN(sizeof(struct ipt_entry_match)) + m->size;
+					m->m = xcalloc(1, size);
+					m->m->u.match_size = size;
+					strcpy(m->m->u.user.name, m->name);
+					m->init(m->m, &fw.nfcache);
+					opts = merge_options(opts, m->extra_opts, 
+							&m->option_offset);
+					break;
+				case 1:
+					if (optarg[0] == '!' && optarg[1] == '\0') {
+						if (invert)
+							exit_error(PARAMETER_PROBLEM,
+									"multiple consecutive ! "
+									"not allowed");
+						invert = TRUE;
+						optarg[0] = '\0';
+						continue;
+					}
+					printf("Bad argument `%s'\n", optarg);
+					exit(1);
+
+				default: 
+					for (m = iptables_matches; m; m = m->next) {
+						if (!m->used)
+							continue;
+						if (m->parse(c - m->option_offset,
+								d->extension, invert,
+								&m->mflags,
+								&fw, &fw.nfcache, &m->m))
+							break;
+					}
+					break;
+			}
+		}
+	}
+	for (m = iptables_matches; m; m = m->next) {
+		if (!m->used)
+			continue;
+		m->final_check(m->mflags);
+	}
+
 	target->final_check(target->tflags);
 	*e = generate_entry(&fw, iptables_matches, target->t);
 
@@ -1318,20 +1394,20 @@ prepare_entry (char *chain, char *saddr, char *sport, char *daddr, char *dport,
  * Try to insert rule into kernel return 0 in case all right, 1 otherwise
  */
 static int
-insert_rule (char *chain, char *saddr, char *sport, char *daddr, char *dport,
-		char *proto, char *targ, int rule_num, char *iface)
+//insert_rule (char *chain, char *saddr, char *sport, char *daddr, char *dport,
+//		char *proto, char *targ, int rule_num, char *iface)
+insert_rule(raw_rule_type *d, int rule_num)
 {
 	struct ipt_entry *e = NULL;
 	int ret=1;
 
-	if (prepare_entry(chain, saddr, sport, daddr, dport, 
-					    proto, targ, iface, &e)!=0)
+	if (prepare_entry(d, &e)!=0)
 		return (1);
 	if (verbose>1) {
 		printf("Inserting rule\n");
 		print_firewall_line(e, handle);
 	}
-	ret &= iptc_insert_entry(chain, e, rule_num, &handle);
+	ret &= iptc_insert_entry(d->dest, e, rule_num, &handle);
 	free(e);
 	return ret;
 }
@@ -1340,21 +1416,21 @@ insert_rule (char *chain, char *saddr, char *sport, char *daddr, char *dport,
  * Try to atomically replace rule in kernel return 0 in case all right, 1 otherwice
  */
 static int
-replace_rule (char *chain, char *saddr, char *sport, char *daddr, char *dport,
-		char *proto, char *targ, int rule_num, char *iface)
+//replace_rule (char *chain, char *saddr, char *sport, char *daddr, char *dport,
+//		char *proto, char *targ, int rule_num, char *iface)
+replace_rule (raw_rule_type *d, int rule_num)
 {
 	struct ipt_entry *e = NULL;
 	int ret=1;
 
-	if (prepare_entry(chain, saddr, sport, daddr, dport, proto, 
-			targ, iface, &e)!=0)
+	if (prepare_entry(d, &e)!=0)
 		return (1);
 
 	if (verbose>1) {
-		printf("Replacing rule %d in '%s'\n", rule_num, chain);
+		printf("Replacing rule %d in '%s'\n", rule_num, d->dest);
 		print_firewall_line(e, handle);
 	}
-	ret &= iptc_replace_entry(chain, e, rule_num, &handle);
+	ret &= iptc_replace_entry(d->dest, e, rule_num, &handle);
 	free(e);
 	return ret;
 }
@@ -1367,22 +1443,21 @@ replace_rule (char *chain, char *saddr, char *sport, char *daddr, char *dport,
  * append_rule () modified to pass struct iface_struct
  * parse_tcp_ports() & parse_udp_ports, correctly parses xxxx:xxxx style port rules
  */
-
 static int
-append_rule (char *chain, char *saddr, char *sport, char *daddr, char *dport, 
-		char *proto, char *targ, char *iface)
+//append_rule (char *chain, char *saddr, char *sport, char *daddr, char *dport, 
+//		char *proto, char *targ, char *iface)
+append_rule (raw_rule_type *d)
 {
 	struct ipt_entry *e = NULL;
 	
-	if (prepare_entry(chain, saddr, sport, daddr, dport, proto, 
-			targ, iface, &e)!=0)
+	if (prepare_entry(d, &e)!=0)
 		return (1);
-
+	
 	if (verbose>1) {
-		printf("Appending rule to chain '%s'\n", chain);
+		printf("Appending rule to chain '%s'\n", d->dest);
 		print_firewall_line(e, handle);
 	}
-	if (!iptc_append_entry(chain, e, &handle)) {
+	if (!iptc_append_entry(d->dest, e, &handle)) {
 		fprintf(stderr, "iptables: %s\n", iptc_strerror(errno));
 		return (1);
 	}
@@ -1391,15 +1466,15 @@ append_rule (char *chain, char *saddr, char *sport, char *daddr, char *dport,
 }
 
 static int
-delete_rule (char *chain, char *saddr, char *sport, char *daddr, char *dport,
-		char *proto, char *targ, char *iface)
+//delete_rule (char *chain, char *saddr, char *sport, char *daddr, char *dport,
+//		char *proto, char *targ, char *iface)
+delete_rule (raw_rule_type *d)
 {
 	struct ipt_entry *e = NULL;
 	unsigned char *mask = NULL;
 	int ret=1;
 
-	if (prepare_entry(chain, saddr, sport, daddr, dport, proto, 
-			targ, iface, &e)!=0)
+	if (prepare_entry(d, &e)!=0)
 		return (1);
 
 	if (verbose>1) {
@@ -1408,7 +1483,7 @@ delete_rule (char *chain, char *saddr, char *sport, char *daddr, char *dport,
 	}
 
 	mask = make_delete_mask(e);
-	ret &= iptc_delete_entry(chain, e, mask, &handle);
+	ret &= iptc_delete_entry(d->dest, e, mask, &handle);
 	free(e);
 	return ret;
 }
@@ -1421,10 +1496,6 @@ delete_num_rule (char *chain, int num)
 	int ret = 1;
 
 	mask = make_delete_mask(e);
-//	if (!iptc_delete_num_entry(chain, num, &handle)) {
-//		fprintf(stderr, "iptables: %s\n", iptc_strerror(errno));
-//		return (1);
-//	}
 	ret &= iptc_delete_num_entry(chain, num, &handle);
 	free(e);
 	return ret;
@@ -1460,33 +1531,6 @@ setup_tables(void)
 			fprintf(stderr, "iptables: %s\n", iptc_strerror(errno));
 			return (1);
 		}
-
-	if ((!is_classic)) {
-		if (!iptc_is_chain("ipac~c_fi", handle))
-			if (!iptc_create_chain("ipac~c_fi", &handle)) {
-				fprintf(stderr, "iptables: %s\n", 
-							iptc_strerror(errno));
-				return (1);
-			}
-		if (!iptc_is_chain("ipac~c_fo", handle))
-			if (!iptc_create_chain("ipac~c_fo", &handle)) {
-				fprintf(stderr, "iptables: %s\n", 
-							iptc_strerror(errno));
-				return (1);
-			}
-		if (!iptc_is_chain("ipac~c_i", handle))
-			if (!iptc_create_chain("ipac~c_i", &handle)) {
-				fprintf(stderr, "iptables: %s\n", 
-							iptc_strerror(errno));
-				return (1);
-			}
-		if (!iptc_is_chain("ipac~c_o", handle))
-			if (!iptc_create_chain("ipac~c_o", &handle)) {
-				fprintf(stderr, "iptables: %s\n", 
-							iptc_strerror(errno));
-				return (1);
-			}
-	}
 	return 0;
 };
 
@@ -1515,33 +1559,6 @@ flush_acc_tables(void)
 			fprintf(stderr, "iptables: %s\n", iptc_strerror(errno));
 			return (1);
 		}
-
-	if (!is_classic) {
-		if (iptc_is_chain("ipac~c_fi", handle))
-			if (!iptc_flush_entries("ipac~c_fi", &handle)) {
-				fprintf(stderr, "iptables: %s\n", 
-							iptc_strerror(errno));
-				return (1);
-			}
-		if (iptc_is_chain("ipac~c_fo", handle))
-			if (!iptc_flush_entries("ipac~c_fo", &handle)) {
-				fprintf(stderr, "iptables: %s\n", 
-							iptc_strerror(errno));
-				return (1);
-			}
-		if (iptc_is_chain("ipac~c_i", handle))
-			if (!iptc_flush_entries("ipac~c_i", &handle)) {
-				fprintf(stderr, "iptables: %s\n", 
-							iptc_strerror(errno));
-				return (1);
-			}
-		if (iptc_is_chain("ipac~c_o", handle))
-			if (!iptc_flush_entries("ipac~c_o", &handle)) {
-				fprintf(stderr, "iptables: %s\n", 
-							iptc_strerror(errno));
-				return (1);
-			}
-	}
 // Try to flush our old chains
 	if (access_agent->get_raw_list("iptables", "", &d)) {
 		fprintf(stderr, "access error\n");
@@ -1551,8 +1568,12 @@ flush_acc_tables(void)
 	while(d) {
 		if (strlen(d->name)>7)
 			if (!memcmp(d->name, "%chain%", 7))
-				if (iptc_is_chain(d->name+8, handle))
+				if (iptc_is_chain(d->name+8, handle)) {
 					iptc_flush_entries(d->name+8, &handle);
+					if (verbose>1)
+						fprintf(stderr, "flushing chain '%s'\n", d->name+8);
+				}
+					
 		d=d->next;
 	}
 	free_raw_list(d1);
@@ -1592,29 +1613,6 @@ int iptables_ipac_init(int flag)
 	return 0;
 };
 
-/* 
- * setup auth host - open access to AUTH_SERVER for all clients
- */
-static int
-setup_auth_host()
-{
-	user_list *list;
-
-	access_agent->get_user_list(&list);
-	while(list) {
-		if (strncmp(list->login, "admin", 5)) {
-			if (insert_rule("ipac~c_fo", authhost, "", list->ip, "", 
-							"", "RETURN", 0, ""))
-					return (1);
-			if (insert_rule("ipac~c_fi",  list->ip, "", authhost, "", 
-							"", "RETURN", 0, ""))
-					return (1);
-		}
-		list=list->next;
-	}
-	return 0;
-}
-
 static int 
 setup_rules(void)
 {
@@ -1634,15 +1632,10 @@ setup_rules(void)
 					    me, RUNFILE, strerror(errno));
 		return 1;
 	}
-	d1=d;
-	while(d) 
-	{
+	d1 = d;
+	while(d) {
 		/* Trying to implement hierarchic rules */
-		if (is_classic)
-			strcpy(targ, "");
-		else 
-			strcpy(targ, d->policy);
-		
+		strcpy(d->target, ""); /* no target per default */
 		strcpy(chain, d->dest); // %-)
 
 		/* Are we dealing with new chain? if so create it */
@@ -1653,10 +1646,12 @@ setup_rules(void)
 							"chain name missing\n");
 					return 1;
 				}
-				strcpy(targ, d->name+8);
-				iptc_create_chain(targ, &handle);
-				if (is_classic)
-					fprintf(frunfile, "%s|%%\n", chain);
+				/* set target to this new chain */
+				strcpy(d->target, d->name+8); 
+				if (verbose>1)
+					fprintf(stderr, "creating chain '%s'\n", d->target);
+				iptc_create_chain(d->target, &handle);
+				fprintf(frunfile, "%s|%%%s%%\n", chain, d->target);
 			} else {
 				fprintf(stderr, "error: incorrect symbol %% "
 						"in rule name\n");
@@ -1667,21 +1662,10 @@ setup_rules(void)
 				    (!memcmp(chain+strlen(chain)-4, "~c", 2))) ||
 			    ((strlen(chain)>5) &&
 				    (!memcmp(chain+strlen(chain)-5, "~c", 2)))))
-				if (is_classic || 
-					!strncmp(chain, "ipac", 4) ||
-					!strncmp(chain, "admin", 5) ||
-					((strlen(d->name)>3) && 
-					    !memcmp(d->name+strlen(d->name)-3, 
-								"~st", 3)))
-					fprintf(frunfile,"%s|%s\n", chain, d->name);
+				fprintf(frunfile,"%s|%s\n", chain, d->name);
 				
-		if (is_classic || 
-				!strncmp(chain, "ipac", 4) ||
-				!strncmp(chain, "admin", 5) ||
-				((strlen(d->name)>3) && 
-					!memcmp(d->name+strlen(d->name)-3, "~st", 3)))
-			append_rule(chain, d->snet, d->sport, d->dnet, d->dport,
-						    d->protocol, targ, d->iface);
+		strcpy(d->dest, chain);
+		append_rule(d);
 		d=d->next;
 	}
 	fclose(frunfile);
@@ -1695,6 +1679,8 @@ setup_rules(void)
 int iptables_ipac_set(rule_type **firstrule, int first)
 {
 	unsigned int ref=0;
+	raw_rule_type d;
+	bzero((void *) &d, sizeof(raw_rule_type));
 
 	if (verbose)
 		fprintf(stderr, "Flushing accounting chains..\n");
@@ -1710,37 +1696,30 @@ int iptables_ipac_set(rule_type **firstrule, int first)
 	if (first==1) {
 		iptc_get_references(&ref, "ipac~fi", &handle);
 		if (ref!=0) {
-			delete_rule("OUTPUT", "0/0", "", "0/0", "", "", "ipac~i", "");
-			delete_rule("FORWARD", "0/0", "", "0/0", "", "", "ipac~fi", "");
+			strcpy(d.dest, "OUTPUT"); strcpy(d.snet, "0/0"); strcpy(d.dnet, "0/0");
+			strcpy(d.target, "ipac~i");
+			delete_rule(&d);
+			strcpy(d.dest, "FORWARD"); strcpy(d.target, "ipac~fi");
+			delete_rule(&d);
 		}
 		iptc_get_references(&ref, "ipac~fo", &handle);
 		if ((ref!=0) && (first==1)) {
-			delete_rule("INPUT", "0/0", "", "0/0", "", "", "ipac~o", "");
-			delete_rule("FORWARD", "0/0", "", "0/0", "", "", "ipac~fo", "");
-		}
-		insert_rule("OUTPUT", "0/0", "", "0/0", "", "", "ipac~i", 0, "");
-		insert_rule("INPUT", "0/0", "", "0/0", "", "", "ipac~o", 0, "");
-		insert_rule("FORWARD", "0/0", "", "0/0", "", "", "ipac~fo", 0, "");
-		insert_rule("FORWARD", "0/0", "", "0/0", "", "", "ipac~fi", 0, "");
-		if (!is_classic) {
-			iptc_get_references(&ref, "ipac~c_fi", &handle);
-			if (ref!=0) {
-				delete_rule("OUTPUT", "", "", "", "", "", "ipac~c_i", "");
-				delete_rule("FORWARD", "", "", "", "", "", "ipac~c_fi", "");
-			}
-			iptc_get_references(&ref, "ipac~c_fo", &handle);
-			if (ref!=0) {
-				delete_rule("INPUT", "", "", "", "", "", "ipac~c_o", "");
-				delete_rule("FORWARD", "", "", "", "", "", "ipac~c_fo", "");
-			}
-			insert_rule("OUTPUT", "", "", "", "", "", "ipac~c_i", 0, "");
-			insert_rule("INPUT", "", "", "", "", "", "ipac~c_o", 0, "");
-			insert_rule("FORWARD", "", "", "", "", "", "ipac~c_fi", 0, "");
-			insert_rule("FORWARD", "", "", "", "", "", "ipac~c_fo", 0, "");
+			strcpy(d.dest, "INPUT"); strcpy(d.snet, "0/0"); strcpy(d.dnet, "0/0");
+                        strcpy(d.target, "ipac~o");
+			delete_rule(&d);
+			strcpy(d.dest, "FORWARD"); strcpy(d.target, "ipac~fo");
+			delete_rule(&d);
 		}
+		strcpy(d.dest, "OUTPUT"); strcpy(d.snet, "0/0"); strcpy(d.dnet, "0/0");
+		strcpy(d.target, "ipac~i");
+		insert_rule(&d, 0);
+		strcpy(d.dest, "INPUT"); strcpy(d.target, "ipac~o");
+		insert_rule(&d, 0);
+		strcpy(d.dest, "FORWARD"); strcpy(d.target, "ipac~fo");
+		insert_rule(&d, 0);
+		strcpy(d.dest, "FORWARD"); strcpy(d.target, "ipac~fi");
+		insert_rule(&d, 0);
 	}
-	if (!is_classic && authhost!=NULL && strlen(authhost)>2)
-		setup_auth_host();
 	iptc_commit(&handle);
 	iptables_ipac_init(0);
 	return 0;
@@ -1758,372 +1737,6 @@ iptables_ipac_read(rule_type **firstrule)
 	return read_iptables(runfile, firstrule);
 }
 
-/*
- * Accept packets to/from host with login name
- */ 
-int 
-iptables_ipac_accept(char *login)
-{
-	raw_rule_type *d;
-	int logged_in=0;
-	unsigned int tci=-1,tco=-1,tcfi=-1,tcfo=-1;
-	char *name;
-	FILE *frunfile;
-	char n[MAX_RULE_NAME_LENGTH+10];
-	char n1[MAX_RULE_NAME_LENGTH+10];
-	char serv_name[MAX_RULE_NAME_LENGTH+10];
-
-	strcpy(serv_name, login);
-	name = strsep(&login, " ");
-	while(strsep(&login, " "));
-
-	iptables_ipac_init(0);
-	access_agent->get_raw_list("iptables", name, &d);
-
-
-	strcpy(n, name); strcat(n, "~c_i");
-
-	if (insert_rule(n, "", "", "", "", "", "", 2, "")) {
-		logged_in = 1;
-		delete_num_rule(n, 2);
-	}
-
-	while(d && !memcmp(d->name, "%chain%", 7)) d = d->next; // skip chains
-
-	strcpy(n, name); strcat(n, " ");
-	strcpy(n1, name); strcat(n1, "~st");
-
-	if (!logged_in) {
-		while(lock(LOCKFILE));
-		frunfile = fopen(RUNFILE, "a");
-    		if (!frunfile) {
-            		fprintf(stderr, "%s: opening runfile \"%s\": %s\n",
-                                        me, RUNFILE, strerror(errno));
-	                return 1;
-    		}
-		clear_runfile(name, 1);
-		while (d) {
-			if ((!strncmp(d->name, n, strlen(n))) ||
-				    (!strcmp(d->name, n1))) {
-				if (!(((strlen(d->dest)>4) && 
-					    (!memcmp(d->dest+strlen(d->dest)-4, "~c", 2))) ||
-					    ((strlen(d->dest)>5) &&
-					    (!memcmp(d->dest+strlen(d->dest)-5, "~c", 2))))) {
-					append_rule(d->dest, d->snet, d->sport, d->dnet, 
-				    	    d->dport, d->protocol, "RETURN", d->iface);
-				} else {
-				        append_rule(d->dest, d->snet, d->sport, d->dnet, 
-					    d->dport, d->protocol, "DROP", d->iface);
-				}
-				if (((!strncmp(d->dest+strlen(d->dest)-2, "~i", 2)) ||
-				    (!strncmp(d->dest+strlen(d->dest)-2, "~o", 2)) ||
-				    (!strncmp(d->dest+strlen(d->dest)-3, "~fi", 3)) ||
-				    (!strncmp(d->dest+strlen(d->dest)-3, "~fo", 3))) &&
-				    (strncmp(d->name+strlen(d->name)-3, "~st", 3))) {
-					fprintf(frunfile, "%s|%s\n", d->dest, d->name);
-				}
-			}
-	    		d=d->next;
-		}
-		// delete old stopper
-		strcpy(n1, name); strcat(n1, "~c_fi"); delete_num_rule(n1, 0);
-		strcpy(n1, name); strcat(n1, "~c_fo"); delete_num_rule(n1, 0);
-		strcpy(n1, name); strcat(n1, "~c_i"); delete_num_rule(n1, 0);
-		strcpy(n1, name); strcat(n1, "~c_o"); delete_num_rule(n1, 0);
-		access_agent->get_raw_list("iptables", name, &d);
-		while(d && !memcmp(d->name, "%chain%", 7)) d = d->next;
-		fclose(frunfile);
-		unlock(LOCKFILE);
-	}	
-
-	while(d) {
-		if (!strncmp(d->dest+strlen(d->dest)-4, "~c_i", 4))
-			tci++;
-		if (!strncmp(d->dest+strlen(d->dest)-4, "~c_o", 4))
-			tco++;
-		if (!strncmp(d->dest+strlen(d->dest)-5, "~c_fi", 5))
-			tcfi++;
-		if (!strncmp(d->dest+strlen(d->dest)-5, "~c_fo", 5))
-			tcfo++;
-		if (!strcmp(d->name, serv_name)) {
-			if (!strncmp(d->dest+strlen(d->dest)-4, "~c_i", 4)) {
-				replace_rule(d->dest, d->snet, d->sport, d->dnet,
-				    d->dport, d->protocol, "RETURN", tci, d->iface);
-			} else if (!strncmp(d->dest+strlen(d->dest)-4, "~c_o", 4)) {
-				replace_rule(d->dest, d->snet, d->sport, d->dnet,
-				    d->dport, d->protocol, "RETURN", tco, d->iface);
-			} else if (!strncmp(d->dest+strlen(d->dest)-5, "~c_fi", 5)) {
-				replace_rule(d->dest, d->snet, d->sport, d->dnet,
-				    d->dport, d->protocol, "RETURN", tcfi, d->iface);
-			} else if (!strncmp(d->dest+strlen(d->dest)-5, "~c_fo", 5)) {
-				replace_rule(d->dest, d->snet, d->sport, d->dnet,
-				    d->dport, d->protocol, "RETURN", tcfo, d->iface);
-			}
-		}
-	    	d=d->next;
-	}
-	iptc_commit(&handle);
-	iptables_ipac_init(0);
-	return 0;
-}
-
-static int
-clear_runfile(char *serv_name, int greedy)
-{
-	FILE *frunfile;
-        char runfile_line[MAX_RULE_NAME_LENGTH + 50], *cp;
-        struct runfile_line_type *result, *lastline, *cur;
-	int tmp;
-	
-	frunfile=fopen(RUNFILE, "r");
-	if (!frunfile) {
-		fprintf(stderr, "%s: opening runfile \"%s\": %s\n",
-					    me, RUNFILE, strerror(errno));
-		return 1;
-	}
-	result = NULL; lastline = NULL;
-
-        while(fgets(runfile_line, MAX_RULE_NAME_LENGTH+50, frunfile) != NULL) {
-    		tmp++;
-                cp = strchr(runfile_line, '\n');
-                if (cp)
-                        *cp = '\0';
-                        if (*runfile_line == '#')
-	                        continue;
-                        cur = (struct runfile_line_type *)
-                                    xmalloc(sizeof(struct runfile_line_type));
-	                cur->line = xstrdup(runfile_line);
-                        cur->next = NULL;
-                        if (result == NULL)
-	                        result = cur;
-                        else
-	                        lastline->next = cur;
-                        lastline = cur;
-	}																																						    
-        if (!feof(frunfile)) {
-	        fprintf(stderr, "%s: reading \"%s\": %s\n",
-					me, RUNFILE, strerror(errno));
-		fclose(frunfile);
-		destroy_runfile_lines(result);
-		result = NULL;
-		return 1;
-	}																																																			    
-	fclose(frunfile);
-	frunfile=fopen(RUNFILE, "w");
-	if (!frunfile) {
-		fprintf(stderr, "%s: opening runfile \"%s\": %s\n",
-					    me, RUNFILE, strerror(errno));
-		return 1;
-	}
-	for (cur=result; cur!=NULL; cur=cur->next) {
-		cp = strchr(cur->line, '|');
-		if (cp == 0)
-			continue;
-		if (strcmp(cp+1, serv_name)) {
-			if (greedy) {
-				if (strncmp(cp+1, serv_name, strlen(serv_name)))
-					fprintf(frunfile, "%s\n", cur->line);
-			} else
-				fprintf(frunfile, "%s\n", cur->line);
-		}
-	}
-	fclose(frunfile);
-	destroy_runfile_lines(result);
-	return 0;
-}
-
-/* 
- * Deny packet crossing to/from user login
- * Warning: there are 2 possible cases:
- *	a) login=user_name
- *		in this case we have to close all services
- *	b) login="user_name - service_name"
- *		in such case we only need to close that service
- * there may be some traffic on the counters for this user. drop this info to spool
- */
-int 
-iptables_ipac_deny(char *login)
-{
-	raw_rule_type *d, *d1;
-	int tci=-1,tco=-1,tcfi=-1,tcfo=-1, tmp=0;
-	char *name;
-	char n[MAX_RULE_NAME_LENGTH+10];
-	char n1[MAX_RULE_NAME_LENGTH+10];
-	char serv_name[MAX_RULE_NAME_LENGTH+10];
-	timestamp_t t;
-	FILE *spool;
-	rule_type *rule = NULL, *lastrule = NULL, *firstrule = NULL, *rule1;
-	void *ruletree = NULL;
-	void *node;
-        struct ipt_counters *counters = NULL;
-
-	strcpy(serv_name, login);
-	name = strsep(&login, " ");
-	while(strsep(&login, " "));
-
-	iptables_ipac_init(0);
-	access_agent->get_raw_list("iptables", name, &d);
-
-	while(d && !memcmp(d->name, "%chain%", 7)) d = d->next; // skip chains
-
-	d1 = d;
-
-	// entirely delete
-	if (!strcmp(serv_name, name)) {
-		while(lock(LOCKFILE));
-		while(d) {
-			if (!(((!strncmp(d->dest+strlen(d->dest)-2, "~i", 2)) ||
-			    (!strncmp(d->dest+strlen(d->dest)-2, "~o", 2)) ||
-			    (!strncmp(d->dest+strlen(d->dest)-3, "~fi", 3)) ||
-			    (!strncmp(d->dest+strlen(d->dest)-3, "~fo", 3))) &&
-			    (strncmp(d->name+strlen(d->name)-3, "~st", 3)))) {
-				d = d->next;
-				continue;
-			}
-
-			if (!strncmp(d->dest+strlen(d->dest)-2, "~i", 2)) {
-				tci++;
-				counters = iptc_read_counter(d->dest, tci, &handle);
-				if (!iptc_zero_counter(d->dest, tci, &handle))
-				    	fprintf(stderr, "iptables: %s\n", iptc_strerror(errno));
-			} 
-			if (!strncmp(d->dest+strlen(d->dest)-2, "~o", 2)) {
-				tco++;
-				counters = iptc_read_counter(d->dest, tco, &handle);
-				if (!iptc_zero_counter(d->dest, tco, &handle))
-				    	fprintf(stderr, "iptables: %s\n", iptc_strerror(errno));
-			}
-			if (!strncmp(d->dest+strlen(d->dest)-3, "~fi", 3)) {
-				tcfi++;
-				counters = iptc_read_counter(d->dest, tcfi, &handle);
-				if (!iptc_zero_counter(d->dest, tcfi, &handle))
-				    	fprintf(stderr, "iptables: %s\n", iptc_strerror(errno));
-	        	}
-			if (!strncmp(d->dest+strlen(d->dest)-3, "~fo", 3)) {
-				tcfo++;
-				counters = iptc_read_counter(d->dest, tcfo, &handle);
-				if (!iptc_zero_counter(d->dest, tcfo, &handle))
-				    	fprintf(stderr, "iptables: %s\n", iptc_strerror(errno));
-			}
-			rule = new_rule();
-			strcpy(rule->name, d->name);
-			/* use a binary tree to find rules with same name */
-			node = tsearch(rule, &ruletree, rule_compare);
-			if (*(rule_type **)node != rule) {
-				free(rule);
-				rule=*(rule_type **)node;
-			} else {
-				if (lastrule != NULL)
-					lastrule->next = rule;
-				lastrule = rule;
-				if (firstrule == NULL)
-					firstrule = rule;
-			}
-			rule->pkts += counters->pcnt;
-			rule->bytes += counters->bcnt;
-			d=d->next;
-		}
-		rule = firstrule;
-		rule1 = firstrule;
-		while(rule) {
-			if (rule->pkts == 0) {
-				if (rule == firstrule) {
-					firstrule = rule->next;
-					rule1 = rule->next;
-				} else
-					rule1->next = rule->next;
-				free(rule);
-				rule = rule->next;
-			} else {
-				rule1 = rule;
-				rule = rule->next;
-				tmp++;
-			}
-		}
-		
-		if (tmp>0) {
-			spool=fopen(spoolfile, "a");
-			if (spool == NULL) {
-				fprintf(stderr, "cant open spool file '%s' for append "
-					"some traffic may be lost.\n%s\n", spoolfile, strerror(errno));
-	    			exit (1);
-			}
-
-			time(&t);
-			fprintf(spool, "BILL\n%lu 1\n", t);
-			fprintf(spool, "( %s\n", hostname);
-
-			rule = firstrule;
-			while(rule) {
-				fprintf(spool, "%llu %llu |%s|\n", rule->bytes, rule->pkts, rule->name);
-				rule = rule->next;
-			}
-			fprintf(spool, ")\n\n");
-			fclose(spool);
-		}
-	        free_tree(&ruletree);
-		d = d1;
-
-		strcpy(n1, name); strcat(n1, "~i"); iptc_flush_entries(n1, &handle);
-		strcpy(n1, name); strcat(n1, "~o"); iptc_flush_entries(n1, &handle);		
-		strcpy(n1, name); strcat(n1, "~fi"); iptc_flush_entries(n1, &handle);		
-		strcpy(n1, name); strcat(n1, "~fo"); iptc_flush_entries(n1, &handle);
-		strcpy(n1, name); strcat(n1, "~c_i"); iptc_flush_entries(n1, &handle);
-		strcpy(n1, name); strcat(n1, "~c_o"); iptc_flush_entries(n1, &handle);
-		strcpy(n1, name); strcat(n1, "~c_fi"); iptc_flush_entries(n1, &handle);
-		strcpy(n1, name); strcat(n1, "~c_fo"); iptc_flush_entries(n1, &handle);
-		strcpy(n1, name); strcat(n1, "~st");
-		// find and insert stoppers
-		while(d) {
-			if (!strcmp(d->name, n1))
-				append_rule(d->dest, d->snet, d->sport, d->dnet, 
-				        d->dport, d->protocol, "DROP", NULL);
-			d=d->next;
-		}
-		strcpy(n1, name); strcat(n1, " ");
-		// greedy deletion
-		clear_runfile(n1, 1);
-		unlock(LOCKFILE);
-	}
-
-	d = d1;
-
-	tci=-1; tco=-1; tcfi=-1; tcfo=-1;
-
-	strcpy(n, name); strcat(n, " ");
-	strcpy(n1, name); strcat(n1, "~st");
-
-	while(d) {
-		if (!strncmp(d->dest+strlen(d->dest)-4, "~c_i", 4))
-			tci++;
-		else if (!strncmp(d->dest+strlen(d->dest)-4, "~c_o", 4))
-			tco++;
-		else if (!strncmp(d->dest+strlen(d->dest)-5, "~c_fi", 5))
-			tcfi++;
-        	else if (!strncmp(d->dest+strlen(d->dest)-5, "~c_fo", 5))
-			tcfo++;
-		if (!strcmp(d->name, serv_name)) {
-			if (!strncmp(d->dest+strlen(d->dest)-4, "~c_i", 4)) {
-				replace_rule(d->dest, d->snet, d->sport, d->dnet,
-				    d->dport, d->protocol, "DROP", tci, NULL);
-			} else if (!strncmp(d->dest+strlen(d->dest)-4, "~c_o", 4)) {
-				replace_rule(d->dest, d->snet, d->sport, d->dnet,
-				    d->dport, d->protocol, "DROP", tco, NULL);
-			} else if (!strncmp(d->dest+strlen(d->dest)-5, "~c_fi", 5)) {
-				replace_rule(d->dest, d->snet, d->sport, d->dnet,
-				    d->dport, d->protocol, "DROP", tcfi, NULL);
-			} else if (!strncmp(d->dest+strlen(d->dest)-5, "~c_fo", 5)) {
-				replace_rule(d->dest, d->snet, d->sport, d->dnet,
-				    d->dport, d->protocol, "DROP", tcfo, NULL);
-			}
-		}
-	    	d=d->next;
-	}
-	iptc_commit(&handle);
-	iptables_ipac_init(0);
-	return 0;
-}
-
 int
 iptables_ipac_check(void){
 	int tmp=0;	
@@ -2153,39 +1766,6 @@ flush_remove_chain(char *ch_name)
 	}
 }
 
-
-int
-iptables_ipac_remove_user(char *login)
-{
-	char tmp[MAX_RULE_NAME_LENGTH];
-	
-	strcpy(tmp, login);
-	strcpy(tmp+strlen(tmp), "~fo");
-	flush_remove_chain(tmp);
-	strcpy(tmp, login);
-	strcpy(tmp+strlen(tmp), "~fi");
-	flush_remove_chain(tmp);
-	strcpy(tmp, login);
-	strcpy(tmp+strlen(tmp), "~c_fi");
-	flush_remove_chain(tmp);
-	strcpy(tmp, login);
-	strcpy(tmp+strlen(tmp), "~c_fo");
-	flush_remove_chain(tmp);
-	strcpy(tmp, login);
-	strcpy(tmp+strlen(tmp), "~o");
-	flush_remove_chain(tmp);
-	strcpy(tmp, login);
-	strcpy(tmp+strlen(tmp), "~i");
-	flush_remove_chain(tmp);
-	strcpy(tmp, login);
-	strcpy(tmp+strlen(tmp), "~c_i");
-	flush_remove_chain(tmp);
-	strcpy(tmp, login);
-	strcpy(tmp+strlen(tmp), "~c_o");
-	flush_remove_chain(tmp);
-	return 0;
-}
-
 /*
  * Deny packets from any to any.
  * Make by inserting drop rule to the forward chain
@@ -2194,9 +1774,12 @@ int
 iptables_ipac_alarm(void)
 {
 	int ret;
+	raw_rule_type d;
+	bzero((void *)&d, sizeof(raw_rule_type));
         if (!handle)
                 iptables_ipac_init(0);
-	ret = insert_rule("FORWARD", "", "", "", "", "", "DROP", 0, "");
+	strcpy(d.dest, "FORWARD"); strcpy(d.target, "DROP");
+	ret = insert_rule(&d, 0);
 	iptc_commit(&handle);
 	return ret;
 }
