@@ -1,10 +1,10 @@
 /*
  *
- * $Id: fetchipac.c,v 1.5.2.28 2000/06/28 23:19:26 moritz Exp $
+ * $Id: fetchipac.c,v 1.20 2004/08/24 18:38:56 friedl Exp $
  *
  * Fetch IP accounting stats
  * Copyright (C) 1997 - 2000 Moritz Both
- *           (C) 2001 - 2002 Al Zaharov   
+ *           (C) 2001 - 2003 Al Zakharov
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,15 +46,13 @@
 #include <assert.h>
 #include "ipac.h"
 
-int 		is_classic;
-
 int		dropzero = 0;
 
 int		verbose = 0;
 /** name of the program */
 const char 	*me;
 
-/** the directory with the database. 
+/** the directory with the database.
  *  At least for plain-file and gdbm storage backend - and in any case for
  *  the spool file.
  */
@@ -79,11 +77,11 @@ const acc_agent_t **acc_agents;
 
 /** the current accounting agent */
 const acc_agent_t *acc_agent;
- 
+
 const access_agent_t **access_agents;
 const access_agent_t *access_agent;
 
- 
+
 /** flag for output format. 0=user readable,
  *  1=for machine (ipacsum) */
 int		machine_output_format = 0;
@@ -96,7 +94,6 @@ int storage_opened = 0;
 
 int access_opened = 1;
 
-int is_classic;
 char *storage_method_name = NULL;
 char *acc_agent_name = DEFAULT_AGENT;
 char *access_agent_name = DEFAULT_ACCESS;
@@ -114,7 +111,6 @@ static const char usage_message_first[] =
 "fetchipac [options]\n"
 "options are:\n"
 "-S|--start\tinitial accounting setup: insert rules and chains\n"
-"-c|--classic [yes|no]\t simulate old-styled ipac\n"
 "-C|--config FILE\t specify config file to use\n"
 "-b|--batch\texecute commands which are fed via standard input\n"
 "-H|--hostname\thostname to work with\n"
@@ -138,13 +134,10 @@ static const char usage_message_second[] = "\n"
 "-t|--timestamps [START][,END]\n"
 	"\t\tlist timestamps. If given, start with timestamps START and\n"
 	"\t\t end with timestamp END\n"
-//"-a|--agent\taccounting agent to use (all=all available, one by one)\n"
+"-u|--sum {START][,END]\n"
+	"\t\tprint summary for given timeframe\n"
 "-R|--list-raw\tlist raw accounting rules\n"
-"-L|--login USER\tlogs USER in (in case there is enough cash on it's account\n"
-"-O|--logoff USER\tkicks USER out\n"
-"-u|--user-list\tget userlist\n"
 "-v|--verbose\tadd one level of verbosity\n"
-"-e|--reconfig\treconfigure\n"
 "See fetchipac(8) for further information.\n";
 
 // check existence of chains and rules in kernel
@@ -158,127 +151,10 @@ check_rules(void)
 }
 
 int
-try_to_login(char *login_name)
-{
-	int tmp = 1;
-	user_list *ul=NULL, *ul1=NULL, *ulist=NULL;
-	FILE *sf;
-	char *tmp1;
-	char line[MAX_RULE_NAME_LENGTH];
-
-	tmp = access_agent->login(login_name);
-	
-	if (tmp)
-		return 1;
-
-	while(lock(LOCKFILE));
-	
-	if (!(sf = fopen(STATUSFILE, "r"))) {
-		printf("Can't open status file %s\n", STATUSFILE);
-		return 1;
-	}
-	while(fgets(line, MAX_RULE_NAME_LENGTH, sf)) {
-		ul = new_user();
-                if (ul1 == NULL)      
-                        ulist = ul;
-                else                 
-                        ul1->next = ul;
-		ul1=ul;
-		strcpy(ul->login, line);
-	}
-	fclose(sf);
-	ul=ulist;
-	if (!(sf = fopen(STATUSFILE, "w"))) {
-		printf("Can't open status file %s\n", STATUSFILE);
-		return 1;
-	}
-	while(ul) {
-		tmp1=strchr(ul->login, '|');
-		tmp1[0]='\0';
-		if (!strcmp(ul->login, login_name)) {
-			strcpy(ul->login, login_name);
-			strcpy(ul->login+strlen(ul->login), "|accept\n");
-		}
-		tmp1[0]='|';
-		fprintf(sf, "%s", ul->login);
-		ul=ul->next;
-	}
-	fclose(sf);
-	unlock(LOCKFILE);
-	return tmp;
-}
-
-int
-try_to_logoff(char *login_name)
-{
-	int tmp;
-	user_list *ul=NULL, *ul1=NULL, *ulist=NULL;
-	FILE *sf;
-	char *tmp1;
-	char line[MAX_RULE_NAME_LENGTH];
-
-	tmp = access_agent->logout(login_name, -1);
-	if (tmp)
-		return 1;
-	
-	while(lock(LOCKFILE));
-
-	if (!(sf = fopen(STATUSFILE, "r"))) {
-		printf("Can't open status file %s\n", STATUSFILE);
-		return 1;
-	}
-	while(fgets(line, MAX_RULE_NAME_LENGTH, sf)) {
-		ul = new_user();
-                if (ul1 == NULL)      
-                        ulist = ul;
-                else                 
-                        ul1->next = ul;
-		ul1=ul;
-		strcpy(ul->login, line);
-	}
-	fclose(sf);
-	ul=ulist;
-	if (!(sf = fopen(STATUSFILE, "w"))) {
-		printf("Can't open status file %s\n", STATUSFILE);
-		return 1;
-	}
-	while(ul) {
-		tmp1=strchr(ul->login, '|');
-		tmp1[0]='\0';
-		if (!strcmp(ul->login, login_name)) {
-			strcpy(ul->login, login_name);
-			strcpy(ul->login+strlen(ul->login), "|deny\n");
-		}
-		tmp1[0]='|';
-		fprintf(sf, "%s", ul->login);
-		ul=ul->next;
-	}
-	fclose(sf);
-	unlock(LOCKFILE);
-	return tmp;
-}
-
-int
-get_user_list(void)
-{
-	user_list *u;
-	
-	if(access_agent->get_user_list(&u)) {
-		fprintf(stderr, "access error\n");
-		access_agent->close();
-		return 1;
-	}
-	while (u) {
-		printf("'%s'\t\t'%s'\n", u->login, u->ip);
-		u=u->next;
-	}
-	return 0;
-}
-
-int 
 list_raw(void)
 {
 	raw_rule_type *r;
+	int i;
 
 	if(access_agent->get_raw_list("", "", &r)) {
 		fprintf(stderr, "access error\n");
@@ -286,15 +162,20 @@ list_raw(void)
 		return 1;
 	}
 	printf("rule name            | destination   | iface | proto | "
-			"source              | destination         | direction | policy\n");
+			"source              | destination         | direction | extensions\n");
 	printf("------------------------------------------------------------------"
 			"------------------------------------------------------\n");
 	while(r) {
-		printf("%-20s | %-13s | %-5s | %-5s | %-18s %s | %-18s %s | %-9s | %-8s \n",r->name,
-		    r->dest,r->iface,r->protocol,r->snet,r->sport,r->dnet,
-		    r->dport,r->direction,r->policy);
+		printf("%-20s | %-13s | %-5s | %-5s | %-18s %s | %-18s %s | %-9s | %-8s",r->name,
+			r->dest,r->iface,r->protocol,r->snet,r->sport,r->dnet,
+			r->dport,r->direction,r->target);
+		for (i=0;i<8;i++) {
+			if (r->extension[i])
+				printf(" %-8s ", r->extension[i]);
+		}
+		printf("\n");
 		r = r->next;
-	}	
+	}
 	return 0;
 }
 
@@ -305,53 +186,21 @@ setup_access(int first)
 
 	if(acc_agent->init(0)) {
 		perror("Failed to initialize accounting agent, "
-						    "aborting operations\n");
+			"aborting operations\n");
 		return 1;
 	}
 	if(acc_agent->set(&tmp, first)) {
 		perror("Failed to setup accounting agent, "
-						    "aborting operations\n");
+			"aborting operations\n");
 		return 1;
 	}
 	return 0;
 }
 
-int
-setup_status()
-{
-	FILE *sf;
-	user_list *ul;
-	
-	if(!(sf=fopen(STATUSFILE, "w"))) {
-		printf("Can't open status file %s\n", STATUSFILE);
-		return 1;
-	}
-	if (access_agent->get_user_list(&ul)) return 1;
-	while(ul) {
-		fprintf(sf, "%s|deny\n", ul->login);
-		ul=ul->next;
-	}
-	fclose(sf);
-	return 0;
-}
-	
 int
 setup_agents(void)
 {
-	setup_access(1);
-	if (!is_classic) {
-		setup_status();
-		if (slogin) {
-			user_list *ul;
-			if (access_agent->get_user_list(&ul)) return 1;
-			while(ul) {
-				if (strcmp(ul->login, "admin"))
-					try_to_login(ul->login);
-				ul=ul->next;
-			}
-		}
-	}
-	return 0;
+	return setup_access(1);
 }
 
 /** print the usage message */
@@ -374,9 +223,9 @@ static void set_hostname()
 	
 	if (!hostname) {
 		if (gethostname(hname, 256) != 0) {
-                	strcpy(hname, "(unknown host)");
+			strcpy(hname, "(unknown host)");
 			fprintf(stderr, "%s warning: cant get hostname, "
-                                "using '%s': %s\n",
+				"using '%s': %s\n",
 				me, hname, strerror(errno));
 		} else
 			hostname = xstrdup(hname);
@@ -387,7 +236,7 @@ static void set_hostname()
  * @see storage_methods
  */
 void
-create_storage_methods() 
+create_storage_methods()
 {
 	int i;
 	const storage_method_t **p;
@@ -405,7 +254,7 @@ create_storage_methods()
 /** create array acc_agents
  * @see acc_agents
  */
-void create_acc_agents() 
+void create_acc_agents()
 {
 	int i;
 	const acc_agent_t **p;
@@ -423,7 +272,7 @@ void create_acc_agents()
 /** create array acc_agents
  * @see access_agents
  */
-void create_access_agents() 
+void create_access_agents()
 {
 	int i;
 	const access_agent_t **p;
@@ -457,66 +306,6 @@ static void nice_time(timestamp_t t, char *s)
 	strftime(s, 29, "%c", tmp);
 }
 
-/* 
- * Some work to make sure that all rules are switched on and we can bill
- * for traffic on them.. then decrement limits at database and cut off user in
- * case there are no more bucks
- */
-int 
-check_cash(const data_record_type *data)
-{
-	rule_type *d;
-	double old_cash, new_cash, price;
-	int pay_type;
-	int was_locked=0;
-	char *last_paid;
-
-	was_locked = lock(LOCKFILE);
-	if (was_locked) 
-		unlock(LOCKFILE);
-
-	d = data->firstrule;
-
-	while(d) {
-		if (fnmatch("admin *", d->name, 0)) {
-			pay_type = access_agent->get_pay_type(d->name);
-			if (pay_type != 0) {
-				old_cash = access_agent->get_cash(d->name);
-				new_cash = old_cash;
-				price = access_agent->get_price(d->name);
-				switch (pay_type)
-				{
-					case 1:		// price for a byte
-						new_cash = old_cash -
-							    (d->bytes*price);
-						break;
-					case 15:
-						last_paid = access_agent->
-							get_last_paid(d->name);
-						if (!fnmatch("*mon*", last_paid, 0)||
-						    !fnmatch("*year*", last_paid, 0)) {
-							new_cash = old_cash - 
-									price;
-							access_agent->set_last_paid(
-							    d->name, "1 mons");
-						}
-						break;
-				}
-				if (old_cash!=new_cash)
-    					access_agent->set_cash(d->name, new_cash);
-				// drop connections to that user
-				if (new_cash<=0) {
-					access_agent->logout(d->name, new_cash);
-				}
-			}
-		}
-		d=d->next;
-	}
-	if (was_locked)
-		lock(LOCKFILE);
-	return 0;
-}
-
 int list_timestamps(timestamp_t tstart, timestamp_t tend, char *ahost)
 {
 	timestamp_t *tlist, *t, before, after;
@@ -533,7 +322,7 @@ int list_timestamps(timestamp_t tstart, timestamp_t tend, char *ahost)
 			&before, &after, ahost);
 	
 	if (n > 0) {
-		printf(machine_output_format ? "%d\n" 
+		printf(machine_output_format ? "%d\n"
 				: "number of timestamps found: %d\n", n);
 		if (before != (timestamp_t)-1) {
 			if (machine_output_format)
@@ -569,27 +358,15 @@ int list_timestamps(timestamp_t tstart, timestamp_t tend, char *ahost)
 			}
 		}
 	} else
-		printf(machine_output_format ? "0\n" 
+		printf(machine_output_format ? "0\n"
 				: "number of timestamps found: 0\n");
 	if (n > 0)
 		free(tlist);
 	return n>-1 ? 0 : 1;
 }
 
-/* signal handler for SIGALRM: close the database backend to avoid
- * locking it too long.
- * commented out, because it introduces more problems that it solves (Manfred Weihs)
- */
-/*static void alarm_closedb(int signo)
-{
-	db_closed_by_timer = 1;
-	alarm(0);
-	storage_method->close();
-}
-*/
-
 /* print a record on file descriptor f */
-int 
+int
 print_records(FILE *f, int n, const data_record_type *dr)
 {
 	char buf[256];
@@ -602,23 +379,23 @@ print_records(FILE *f, int n, const data_record_type *dr)
 	}
 	
 	for (i=0; i<n; i++, dr++) {
-//		if (is_classic)
-        		fprintf(f, "%s%lu ", machine_output_format ? "ADD\n":"timstamp: ",
-				    dr->timestamp);
-//		else
-//			fprintf(f, "%s%lu ", machine_output_format ? "BILL\n":"timstamp: ",
-//				    dr->timestamp);
-        	if (machine_output_format == 0) {
-		        nice_time(dr->timestamp, buf);
-        		fprintf(f, "(%s) ", buf);
-	        }
-	        fprintf(f, "%s1\n", machine_output_format ? "":
+		fprintf(f, "%s%lu ", machine_output_format ? "ADD\n":"timstamp: ",
+			  dr->timestamp);
+		if (machine_output_format == 0) {
+			nice_time(dr->timestamp, buf);
+			fprintf(f, "(%s) ", buf);
+		}
+		fprintf(f, "%s1\n", machine_output_format ? "":
 			"number of records: ");
 		if (machine_output_format == 0 && n>1)
 			fprintf(f, "  record number: 1\n");
 		fprintf(f, machine_output_format ?
 				"( %s\n" : "    machine name: %s\n",
-				dr->machine_name);
+				/* FIXME: at the moment it's useless to
+				          print the machine name returned
+				dr->machine_name
+				*/
+				(ahost != NULL) ? ahost : "");
 		for (r = dr->firstrule; r != NULL; r=r->next)
 			fprintf(f, machine_output_format ?
 				"%llu %llu |%s|\n"
@@ -633,58 +410,79 @@ print_records(FILE *f, int n, const data_record_type *dr)
 /* list record data of timestamps on standard input */
 int list_records(char *filter)
 {
-	timestamp_t timestamp_b,timestamp_e;
-	data_record_type *dr;
+	timestamp_t timestamp_b, timestamp_e, *t, *tlist = NULL;
+	data_record_type *dr = NULL;
 	char buf[256], *cp;
-	int n;
-//	struct sigaction act;
-	
-//Why do we have that alarm stuff here? I think that is not necessary.
-//I think storage is never opened here; therefore that "if (!storage_opened) are useless. (Manfred Weihs)
-//        act.sa_handler = alarm_closedb;
-//	sigemptyset(&act.sa_mask);
-//	act.sa_flags = SA_RESTART;
-//	act.sa_restorer = NULL;
+	int n = 0, i, cnt;
 
-//	db_closed_by_timer = 0;
-	if (!storage_opened) {
+	if (!storage_opened)
 		storage_method->open(SM_OPEN_READONLY);
-//		sigaction(SIGALRM, &act, NULL);
-//		alarm(5);
-	}
-	while(fgets(buf, 256, stdin) != NULL)
-	{
-//		alarm(0);
+
+	while(fgets(buf, 256, stdin) != NULL) {
 		cp = buf;
 		if (*cp == '+' || *cp == '*' || *cp == '-')
 			cp++;
 		while (*cp!=0 && *cp==' ')
 			cp++;
 		timestamp_b = strtoul(cp, &cp, 0);
-                if (*cp == '-')
-                { //we also have an end timestamp
-                        cp++;
-        		timestamp_e = strtoul(cp, NULL, 0);
+		if (*cp == '-') { //we also have an end timestamp
+			cp++;
+			timestamp_e = strtoul(cp, NULL, 10);
 		}
-                else
-                        timestamp_e = 0;
-//		if (db_closed_by_timer)
-//		{
-//			storage_method->open(SM_OPEN_READONLY);
-//			db_closed_by_timer = 0;
-//		}
-		n = storage_method->get_records(timestamp_b, timestamp_e, &dr, filter);
-//		if (!storage_opened)
-//			alarm(5);
-		print_records(stdout, n, dr);
+		else
+			timestamp_e = 0;
 
-		if (n>0)
+		if (timestamp_e && !strcmp(storage_method->name, "postgre")) {
+			/* for postgres storage and query for a timestamp range
+			   do some memory optimizations */
+			cnt = storage_method->list_timestamps(timestamp_b, timestamp_e, &tlist,
+								NULL, NULL, NULL);
+			
+			for (i=0, t=tlist; i<cnt; i++, t++) {
+				n = storage_method->get_records(*t, 0, &dr, filter);
+				print_records(stdout, n, dr);
+
+			}
+			if (tlist) {
+				free (tlist);
+				tlist = NULL;
+			}
+		} else {
+			/* all other storage methodes do it the default way */
+			n = storage_method->get_records(timestamp_b, timestamp_e, &dr, filter);
+			print_records(stdout, n, dr);
+		}
+		
+		/* free memory
+		   FIXME: in the future all backends should clear it's data at store_close() time */
+		if (strcmp(storage_method->name, "postgre") 
+		 && strcmp(storage_method->name, "sqlite") 
+		 && strcmp(storage_method->name, "mysql") 
+		 && (n>0))
+			/* only for storage backends that don't already do free their own data 
+			   we have to do it now */
 			free_data_record_type_array(dr, n);
 		
 	}
-//	alarm(0);
-//	if (!storage_opened)
-		storage_method->close();
+	storage_method->close();
+	fflush(stdout);
+	return 0;
+}
+
+int print_summary(timestamp_t tstart, timestamp_t tend,
+			char *ahost, char *filter)
+{
+	data_record_type *dr;
+	int n;
+
+	storage_method->open(SM_OPEN_READONLY);
+	n = storage_method->get_summary(tstart, tend, &dr, filter);
+	print_records(stdout, n, dr);
+
+	if (n > 0)
+		free_data_record_type_array(dr, n);
+
+	storage_method->close();
 	fflush(stdout);
 	return 0;
 }
@@ -693,6 +491,7 @@ int list_records(char *filter)
 int spool_record(const data_record_type *dr)
 {
 	FILE *f;
+	char *ahost_saved;
 	int mof;
 	int ret;
 
@@ -705,6 +504,10 @@ int spool_record(const data_record_type *dr)
 
 	mof = machine_output_format;
 	machine_output_format = 1;
+	/* we have to use the ahost - see storage/postgre for details why
+	   the hostname in the rules itself is ignored */ 
+	ahost_saved = ahost;
+	ahost = dr->machine_name;
 	ret = 0;
 	if (print_records(f, 1, dr) != 0) {
 		fprintf(stderr, "%s: cant write data record to spool file \""
@@ -714,6 +517,7 @@ int spool_record(const data_record_type *dr)
 	}
 	fputs("\n", f);
 	fclose(f);
+	ahost = ahost_saved;
 	machine_output_format = mof;
 	return ret;
 }
@@ -774,7 +578,7 @@ void unspool()
 			newf = fopen(spoolfile_new, "w");
 			if (newf == NULL) {
 				fprintf(stderr, "%s: cant open new spool file "
-						"\"%s\": %s\n", me, 
+						"\"%s\": %s\n", me,
 						spoolfile_new,
 						strerror(errno));
 				fclose(f);
@@ -783,7 +587,7 @@ void unspool()
 			do {
 				if (fputs(line, newf) == EOF) {
 					fprintf(stderr, "%s: cant write to file"
-						"\"%s\": %s\n", me, 
+						"\"%s\": %s\n", me,
 						spoolfile_new,
 						strerror(errno));
 					fclose(newf);
@@ -793,7 +597,7 @@ void unspool()
 			} while(fgets(line, 256, f) != NULL);
 			if (ferror(f)) {
 				fprintf(stderr, "%s: error reading from spool "
-						"file: %s", me, 
+						"file: %s", me,
 						strerror(errno));
 				fclose(f);
 				fclose(newf);
@@ -803,7 +607,7 @@ void unspool()
 			fflush(newf);
 			if (ferror(newf)) {
 				fprintf(stderr, "%s: error writing to new spool"
-						" file: %s", me, 
+						" file: %s", me,
 						strerror(errno));
 				fclose(newf);
 				return;
@@ -811,7 +615,7 @@ void unspool()
 			fclose(newf);
 			if (rename(spoolfile_new, spoolfile) != 0) {
 				fprintf(stderr, "%s: error renaming \"%s\" to "
-						"\"%s\": %s\n", me, 
+						"\"%s\": %s\n", me,
 						spoolfile_new, spoolfile,
 						strerror(errno));
 			}
@@ -843,10 +647,7 @@ int batchmode()
 int main(int argc, char **argv)
 {
 	rule_type *firstrule = NULL;
-	char *login_name = NULL;
 
-	void *node = NULL;
-	void *usertree = NULL;	
 	int c;
 	timestamp_t t, tstart=0, tend=0;
 	data_record_type data_record;
@@ -862,25 +663,21 @@ int main(int argc, char **argv)
 
 	static struct option long_options[] = {
 		{ "agent",	    required_argument,	NULL,	'a'	},
-		{ "login",	    required_argument,	NULL,	'L'	},
-		{ "logoff",	    required_argument,	NULL,	'O'	},
 		{ "config",	    required_argument,	NULL,	'C'	},
-		{ "filter",         required_argument,	NULL,	'f'	},
-		{ "start",          no_argument,	NULL,	'S'	},
+		{ "filter",	    required_argument,	NULL,	'f'	},
+		{ "start",	    no_argument,	NULL,	'S'	},
 		{ "list-raw",       no_argument,	NULL,	'R'	},
-		{ "user-list",	    no_argument,	NULL,	'u'	},
 		{ "ahost",	    required_argument,  NULL,   'H'     },
 		{ "batch",	    no_argument,	NULL,	'b'	},
 		{ "directory",	    required_argument,	NULL,	'd'	},
 		{ "help",	    no_argument,	NULL,	'h'	},
 		{ "machine-output-format", no_argument,	NULL,	'm'	},
 		{ "records",	    optional_argument,	NULL,	'r'	},
-		{ "classic",	    required_argument,	NULL,	'c'	},
 		{ "storage-method", required_argument,	NULL,	's'	},
 		{ "timestamps",	    optional_argument,	NULL,	't'	},
 		{ "verbose",	    no_argument,	NULL,	'v'	},
 		{ "version",	    no_argument,	NULL,	'h'	},
-		{ "reconfig",	    no_argument,	NULL,   'e'	},
+		{ "sum",	    optional_argument,	NULL,	'u'	},
 		{ NULL,		    0,			NULL,	0	}
 	};
 
@@ -888,11 +685,6 @@ int main(int argc, char **argv)
 	mode = 0;	/* fetch data */
 	strncpy(conffile, CONFFILE, sizeof(conffile)-1); // default config
 
-	if (!memcmp(EMULATE_CLASSIC, "no", 2))
-		is_classic = 0;
-	else
-		is_classic = 1;
-
 	/* create array storage_methods */
 	create_storage_methods();
 	create_acc_agents();
@@ -901,7 +693,7 @@ int main(int argc, char **argv)
 	/* parse command line */
 	while(1)
 	{
-		c = getopt_long(argc, argv, "a:L:O:C:f:SRuH:bd:vhmr::c:s:t::e", 
+		c = getopt_long(argc, argv, "a:L:O:C:f:SRUH:bd:vhmu::r::c:s:t::e",
 				long_options, NULL);
 		if (c==EOF)
 			break;
@@ -923,29 +715,12 @@ int main(int argc, char **argv)
 			case 'f':
 				filter = xstrdup(optarg);
 				break;
-			case 'c':
-				if (memcmp(optarg, "yes", 3))
-					is_classic = 0;
-				else
-					is_classic = 1;
-				break;
 			case 'C':
 				strncpy(conffile, optarg, sizeof(conffile)-1);
 				break;
-			case 'L':
-				login_name = optarg;
-				mode = 6;
-				break;
-			case 'O':
-				login_name = optarg;
-				mode = 7;
-				break;
 			case 'R':
 				mode = 4;
 				break;
-			case 'u':
-				mode = 8;
-				break;
 			case 'd':
 				datadir = optarg;
 				break;
@@ -964,14 +739,10 @@ int main(int argc, char **argv)
 			case 's':
 				storage_method_name = optarg;
 				break;
-			case 'e':
-				mode = 9;
-				break;
 			case 't':
 				mode = 1; /* list timestamps */
-				if (optarg != NULL)
-				{
-					tstart=strtoul(optarg, &cp, 0);
+				if (optarg != NULL) {
+					tstart = strtoul(optarg, &cp, 0);
 					if (*cp == ',')
 						tend=strtoul(++cp, NULL, 0);
 					else
@@ -981,6 +752,19 @@ int main(int argc, char **argv)
 					tend = time(NULL);
 				}
 				break;
+			case 'u':
+				mode = 10; /* get summary */
+				if (optarg != NULL) {
+					tstart = strtoul(optarg, &cp, 0);
+					if (*cp == ',')
+						tend = strtoul(++cp, NULL, 0);
+					else
+						tend = time(NULL);
+				} else {
+					tstart = 0;
+					tend = time(NULL);
+				}
+				break;
 			case '?':
 				fprintf(stderr, "%s: unknown option\n",	me);
 				exit(1);
@@ -1041,7 +825,6 @@ int main(int argc, char **argv)
 		fprintf(stderr, "%s: error while opening access database\n", me);
 		access_opened = 0;
 		fprintf(stderr, "cant operate while database access is absent\n");
-		acc_agent->alarm();
 		return 1;
 	}
 	spoolfile = xmalloc(strlen(datadir) + sizeof(SPOOLFILE) + 10);
@@ -1060,37 +843,17 @@ int main(int argc, char **argv)
 			return list_records(filter);
 		case 3:
 			return batchmode();
-		case 4: 
+		case 4:
 			return list_raw();
 		case 5:
 			return setup_agents();
-		case 6:
-			if (!is_classic) {
-				if (try_to_login(login_name)) {
-					printf("Login failed\n");
-					return 1;
-				} else
-					return 0;
-			} else {
-				fprintf(stderr, "Login is not supported in "
-							    "classic mode\n");
-				return 1;
-			}
-		case 7:
-			if (!is_classic)
-				return try_to_logoff(login_name);
-			else {
-				fprintf(stderr, "Logoff is not supported in"
-							    " classic mode\n");
-				return 1;
-			}
-		case 8:
-			return get_user_list();
+		case 10:
+			return print_summary(tstart, tend, ahost, filter);
 	}
 
 // fetch data
 	if (acc_agent->init(0)) {
-		printf("Error while initializing accounting agent\n");
+		fprintf(stderr, "Error while initializing accounting agent\n");
 		return 1;
 	}
 	if (check_rules()) {
@@ -1131,7 +894,7 @@ int main(int argc, char **argv)
 						&before, &after);
 			if (tmp>0) t++;
 		}
-*/		
+*/
 		if (acc_agent->read(&firstrule) == 0) {
 			data_record.timestamp = t;
 			data_record.firstrule = firstrule;
@@ -1145,8 +908,8 @@ int main(int argc, char **argv)
 				while (rule1) {
 					if (rule1->pkts==0)
 						rule2->next=rule1->next;
-					else 
-						rule2=rule1;	
+					else
+						rule2=rule1;
 					rule1=rule1->next;
 				}
 			}
@@ -1157,8 +920,6 @@ int main(int argc, char **argv)
 				else
 					fprintf(stderr, "%s: storage reports error on store record\n", me);
 			}
-			if (!is_classic && access_opened)
-				check_cash(&data_record);
 
 			/* if our data could not be stored for some reason,
 			 * we put it in ASCII representation into a spool
@@ -1175,55 +936,6 @@ int main(int argc, char **argv)
 		}
 
 		unlock(LOCKFILE);
-		/* reconfigure, if needed. supported only for neoclassic mode
-		 * we have to store data before reconfiguring
-		 * warning, there are posible traffic loss doing reconfigure
-		 * althought it is very small possibility. in case of data
-		 * blame me loss don't
-		 */
-		if ((mode==9) && (!is_classic) && (access_opened)) {
-			user_list *ul=NULL, *ul1=NULL, *ulist=NULL;
-			FILE *sf;
-			char line[MAX_RULE_NAME_LENGTH];
-
-			if (!(sf = fopen(STATUSFILE, "r"))) {
-				printf("Can't open status file %s\n", STATUSFILE);
-				return 1;
-			}
-			while(fgets(line, MAX_RULE_NAME_LENGTH, sf)) {
-				ul = new_user();
-		                if (ul1 == NULL)
-		                        ulist = ul;
-		                else
-		                        ul1->next = ul;
-				ul1=ul;
-				strcpy(ul->login, line);
-			}
-			fclose(sf);
-			access_agent->get_user_list(&ul1);
-
-			while (ul1) {
-				node = tsearch(ul1->login, &usertree, rule_compare);
-				ul1=ul1->next;
-			}
-			ul=ulist;
-			setup_access(0);
-			setup_status();
-			while(ul) {
-				char *tmp;
-				tmp = strchr(ul->login, '|');
-				if (tmp) {
-					tmp[0]='\0';
-					if (!tfind(ul->login, &usertree, 
-								rule_compare))
-						acc_agent->remove_user(ul->login);
-					else
-						if (tmp[1]=='a')
-							try_to_login(ul->login);
-				}
-				ul=ul->next;
-			}
-		}
 	}
 	if (access_opened)
 		access_agent->close();
