@@ -2,7 +2,7 @@
 #define IPAC_H_INCLUDED
 /*
  *
- * $Id: ipac.h,v 1.5.2.18 2000/06/29 20:30:36 moritz Exp $
+ * $Id: ipac.h,v 1.7 2004/06/27 22:08:54 friedl Exp $
  *
  * ipac global header file for c porgrams
  * Copyright (C) 1997 - 2000 Moritz Both
@@ -75,9 +75,7 @@ extern int storage_opened;
 
 extern int access_opened;
 
-/* flag to indicate work method */
-extern int is_classic;
-
+/* the database configuration */
 extern char *dbname;
 extern char *dbport;
 extern char *dbhost;
@@ -151,15 +149,16 @@ struct Sraw_rule_type {
 // destination port	
 	char		dport[20];
 // protocol
-	char		protocol[6];
+	char		protocol[8];
 // interface name	
 	char		iface[10];
 // (destination)? in/out and many-many more
 	char		dest[MAX_RULE_NAME_LENGTH+1];
-// ''/'-j REJECT'/'-j RETURN'/'-j QUEUE'
-	char		policy[10];
+// target chain (used for custom chains) or target '-j REJECT'/'-j RETURN'/'-j QUEUE'
+	char		target[MAX_RULE_NAME_LENGTH+1];
 // direction of packet: strictly in/out
-	char		direction[4];	
+	char		direction[4];
+	char 		*extension[16];
 /** next item in a singly linked list */
 	struct Sraw_rule_type *next;
 };
@@ -314,11 +313,7 @@ struct Sacc_agent_t {
 	int (*init)(int flag);
 	int (*set)(rule_type **firstrule, int first);
 	int (*read)(rule_type **firstrule);
-	int (*deny)(char *login_name);
-	int (*accept)(char *login_name);
 	int (*check)(void);
-	int (*remove_user)(char *login);
-	int (*alarm)(void);
 };
 
 typedef struct Sacc_agent_t acc_agent_t;
@@ -460,6 +455,11 @@ struct Sstorage_method_t {
 	int (*get_records)(timestamp_t timestamp_s, timestamp_t timestamp_e,
 			data_record_type **data, char *filter);
 
+	/** - a function to retrieve summary from a given time interval
+	 */
+	int (*get_summary)(timestamp_t timestamp_s, timestamp_t timestamp_e,
+			data_record_type **data, char *filter);
+
 	/** pointer to a function to delete a record.
 	 * Deletes the record(s) specified by timestamp.
 	 * @param timestamp the timestamp of the record(s) to delete.
