@@ -1,6 +1,6 @@
 /*
  *
- * $Id: rule.c,v 1.2.2.2 2000/04/26 11:11:57 moritz Exp $
+ * $Id: rule.c,v 1.5 2003/11/08 03:22:33 friedl Exp $
  *
  * Copyright (C) 1997 - 2000 Moritz Both
  * 			2001 Al Zaharov
@@ -48,7 +48,7 @@ raw_rule_type
 {
 	raw_rule_type *rule = (raw_rule_type *)xcalloc(sizeof(raw_rule_type), 1);
 	// PFM!! :-O
-//	memset(rule, 0, sizeof(raw_rule_type));
+	bzero(rule, sizeof(raw_rule_type));
 	return rule;
 }
 
@@ -62,21 +62,33 @@ int rule_compare(const void *p1, const void *p2)
  */
 void free_data_record_type_array(data_record_type *data, int n)
 {
+	/* FIXME: the following variables are only needed in the loop
 	int i;
 	data_record_type *dr;
 	rule_type *r;
+	*/
 
+	/* FIXME: there's no usefull code here so don't do the loop
 	for (i=0, dr=data; i<n; i++, dr++)
 	{
+	*/
+		/* FIXME: we currently don't store the machine name
 		if (dr->machine_name != NULL)
 			free(dr->machine_name);
+		*/
+		/* FIXME: should delegate this to the storage module
+		          as it knows better how to free this
+		          
 		while(dr->firstrule != NULL)
 		{
 			r = dr->firstrule->next;
 			free(dr->firstrule);
 			dr->firstrule = r;
 		}
+		*/
+	/*
 	}
+	*/
 	free(data);
 }
 
