@@ -1,6 +1,6 @@
 /*
  *
- * $Id: gdbm.c,v 1.1.2.12 2000/06/23 09:22:45 moritz Exp $
+ * $Id: gdbm.c,v 1.4 2004/04/18 20:26:12 friedl Exp $
  *
  * gdbm backend to ipac
  * Copyright (C) 1997 - 2000 Moritz Both
@@ -114,10 +114,12 @@ static const storage_method_t interface_entry = {
 	gdbm_ipac_store_record,
 	gdbm_ipac_list_timestamps,
 	gdbm_ipac_get_records,
+	gdbm_ipac_get_summary,
 	gdbm_ipac_delete_record,
 	gdbm_ipac_close
 };
 
+
 /* the interface function */
 const storage_method_t *ipac_sm_interface_gdbm() {
 	return &interface_entry;
@@ -467,19 +469,11 @@ static void gdbm_init()
 
 	if (init_done == 0)
 	{
-		/* check record lengths. This is the only spot where these
-		 * values are hard-coded.
+		/* removed hardcoded check for variable length which breaks 
+		 * on 64 bit machines
+		 * 18.04.2004 - Thomas Zehetbauer <thomasz@hostmaster.org>
 		 */
-		if (sizeof(gdbm_timestamp) != 5
-				|| sizeof(gdbm_data_value_header_t) != 16
-				|| sizeof(gdbm_data_value_record_t) != 20
-				|| sizeof(gdbm_lookup_t) != 8)
-		{
-			fprintf(stderr, "%s: bad record sizes in internal data "
-					"definition structures\n", me);
-			exit(1);
-		}
-
+		 
 		/* set the file names. */
 		i = strlen(datadir) + 1;
 		filename_machine_db =(char *)xmalloc(i+sizeof(GDBM_MACHINE_DB));
@@ -1112,6 +1106,13 @@ int gdbm_ipac_get_record(timestamp_t timestamp, data_record_type *data)
 	return n;
 }
 
+int gdbm_ipac_get_summary(timestamp_t timestamp_s, timestamp_t timestamp_e,
+				data_record_type **data)
+{
+	fprintf(stderr, "gdbm storage does not support summary request\n");
+	return -1;
+}
+
 int gdbm_ipac_get_records(timestamp_t timestamp_s, timestamp_t timestamp_e,
 				data_record_type **data)
 {
