@@ -2,7 +2,7 @@
 #define IPACGDBM_H_INCLUDED
 /*
  *
- * $Id: ipac_gdbm.h,v 1.1.2.5 2000/06/24 19:58:59 moritz Exp $
+ * $Id: ipac_gdbm.h,v 1.3 2003/09/28 11:24:59 kaiser13 Exp $
  *
  * ipac gdbm storage backend header file
  * Copyright (C) 2000 Moritz Both
@@ -100,6 +100,8 @@ int gdbm_ipac_list_timestamps(timestamp_t start, timestamp_t end,
 		timestamp_t *just_after, char *);
 int gdbm_ipac_get_records(timestamp_t timestamp_s, timestamp_t timestamp_e,
 		data_record_type **data);
+int gdbm_ipac_get_summary(timestamp_t timestamp_s, timestamp_t timestamp_e,
+                data_record_type **data);
 int gdbm_ipac_delete_record(timestamp_t timestamp);
 void gdbm_ipac_close();
 
