@@ -1,6 +1,6 @@
 /*
  *
- * $Id: plain-file.c,v 1.1.2.9 2000/06/13 13:07:20 moritz Exp $
+ * $Id: plain-file.c,v 1.3 2003/09/28 11:24:59 kaiser13 Exp $
  *
  * old "plain file" backend to fetchipac
  * Copyright (C) 1997 - 2000 Moritz Both
@@ -42,6 +42,8 @@ int plain_file_ipac_list_timestamps(timestamp_t start, timestamp_t end,
 		timestamp_t *just_after, char *);
 int plain_file_ipac_get_records(timestamp_t timestamp_s,timestamp_t timestamp_e,
 		data_record_type **data);
+int plain_file_ipac_get_summary(timestamp_t timestamp_s,timestamp_t timestamp_e,
+                data_record_type **data);
 int plain_file_ipac_get_raw_list(char *ag_name, char *login, 
 		raw_rule_type **data);
 int plain_file_ipac_delete_record(timestamp_t timestamp);
@@ -53,6 +55,7 @@ const storage_method_t interface_entry = {
 	plain_file_ipac_store_record,
 	plain_file_ipac_list_timestamps,
 	plain_file_ipac_get_records,
+	plain_file_ipac_get_summary,
 	plain_file_ipac_delete_record,
 	plain_file_ipac_close
 };
@@ -162,13 +165,13 @@ int plain_file_ipac_list_timestamps(timestamp_t start, timestamp_t end,
 	timestamp_t ts, *ts_list=NULL;
 	int ts_list_len = 0;
 	int ts_list_used = 0;
-
+/*
 	if (!ahost) {
 		fprintf(stderr, "%s: accounting hosts is not supported by "
 				"'plain-file' storage method\n", me);
 		return -1;
 	}
-	
+*/	
 	d = opendir(datadir);
 	if (d == NULL) {
 		fprintf(stderr, "%s: cant open directory \"%s\": %s\n", me,
@@ -375,6 +378,14 @@ int plain_file_ipac_get_record(timestamp_t timestamp,
 	return 1;
 }
 
+int plain_file_ipac_get_summary(timestamp_t timestamp_s,timestamp_t timestamp_e,
+                data_record_type **data)
+{
+	fprintf(stderr, "plain file storage does not support summary request\n");
+	return -1;
+}
+
+
 int plain_file_ipac_get_records(timestamp_t timestamp_s,timestamp_t timestamp_e,
 		data_record_type **data)
 {
